#include <SPI.h>
#include <RH_RF95.h>
#include <TinyGPS++.h>
#include <SoftwareSerial.h>
// Library from Mikal Hart
/*
   This sample code demonstrates the normal use of a TinyGPS++ (TinyGPSPlus) object.
   It requires the use of SoftwareSerial, and assumes that you have a
   4800-baud serial GPS device hooked up on pins 4(rx) and 3(tx).
*/
const int GPS_ONOFFPin = A3;
const int GPS_SYSONPin = A2;
static const int RXPin = A1, TXPin = A0;
static const uint32_t GPSBaud = 9600;
unsigned long startmillis = 0;
unsigned long startmillis1 = 0;
bool active_tracking = false;
// The TinyGPS++ object
TinyGPSPlus gps;
// The RF95 instance
RH_RF95 rf95;
// The serial connection to the GPS device
SoftwareSerial ss(RXPin, TXPin);
struct Location{
  float latitude = 0;
  float longitude = 0;
  int satellite = 0;};
Location locatin;

void setup()
{ pinMode(GPS_SYSONPin, INPUT);
  digitalWrite(GPS_ONOFFPin, LOW);
  pinMode(GPS_ONOFFPin, OUTPUT);
  delay(100);
  Serial.begin(115200);
  ss.begin(GPSBaud);
  Serial.println("GPS started");
  if (!rf95.init())
   Serial.println("init failed");

}

void loop()
{ if(millis() - startmillis > 10000) // loops every 10 sec
{ startmillis = millis();
  rf95.send((uint8_t*)&locatin, sizeof(Location));
  rf95.waitPacketSent();
  uint8_t buf[RH_RF95_MAX_MESSAGE_LEN];
  uint8_t len = sizeof(buf);
  if (rf95.waitAvailableTimeout(3000))//wait for 3 sec to receive the acknowledgement
      { 
    // Should be a reply message for us now   
        if (rf95.recv(buf, &len))
         {
          Serial.print("got reply: ");
          Serial.println((char*)buf);
          Serial.print("RSSI: ");
          Serial.println(rf95.lastRssi(), DEC);
          if(strcmp((char*)buf,"active") == 0)//check whether received ack is "active"
          {
            active_tracking = true;
            activeupdates();//start GPS updates if acknowledgement is "active"
          }
          }
       else
        {
         Serial.println("recv failed");
         }
      }
     else
     {
        Serial.println("No reply, is rf95_server running?");
     } 
}
 while(digitalRead(GPS_SYSONPin) == HIGH)//turns off GPS
{
    digitalWrite( GPS_ONOFFPin, HIGH );
    delay(5);
    digitalWrite( GPS_ONOFFPin, LOW );
    delay(100);
   Serial.println("Unable to GPS turn off");
}

}
// This custom version of delay() ensures that the gps object
// is being "fed".
static void smartDelay(unsigned long ms)
{
  unsigned long start = millis();
  do 
  {
    while (ss.available())
      gps.encode(ss.read());
  } while (millis() - start < ms);
}

void activeupdates()
{ 
  while(digitalRead(GPS_SYSONPin) == LOW)//turns on GPS
  {
   digitalWrite( GPS_ONOFFPin, HIGH );
    delay(5);
    digitalWrite( GPS_ONOFFPin, LOW );
    delay(100);
  }
  while(active_tracking)
  {
    locatin.latitude = gps.location.lat();
    locatin.longitude = gps.location.lng();
    locatin.satellite = gps.satellites.value();
    Serial.println("GPS data updated");
    if(millis() - startmillis1 > 5000)// sends GPS data every 5sec
     {
     startmillis1 = millis();
     rf95.send((uint8_t*)&locatin, sizeof(Location));// send location data structure
     rf95.waitPacketSent();
     Serial.println(locatin.latitude, 6);
     Serial.println(locatin.longitude, 6);
     Serial.println(locatin.satellite);
     Serial.println("GPS data send");
     uint8_t buf[RH_RF95_MAX_MESSAGE_LEN];
     uint8_t len = sizeof(buf);
     int count = 0;
     if (rf95.waitAvailableTimeout(3000))// waits for 3 seconds to receive the ack
      { 
    // Should be a reply message for us now   
        if (rf95.recv(buf, &len))
         {
          Serial.print("got reply: ");
          Serial.println((char*)buf);
          // Serial.print("RSSI: ") ;
          // Serial.println(rf95.lastRssi(), DEC); 
          if(strcmp((char*)buf,"stop") == 0)//check whether received ack is "stop"
           {
             active_tracking = false;
             break;    
           }   
         }
        else
        {
         Serial.println("recv failed");
        }
      }
     else
     {
        Serial.println("No reply, is rf95_server running?");
        count++;
        if(count >= 5)
        {
         active_tracking = false;
         break; 
        }    
     }  
    }
  Serial.println();
  smartDelay(1000);

  if (millis() > 5000 && gps.charsProcessed() < 10)
    Serial.println(F("No GPS data received: check wiring"));
  }
}
